package com.example.demo.controller;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.*;
import org.springframework.http.client.MultipartBodyBuilder;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/file")
public class FileUploadController {

    @Value("${upload.directory}")
    private String uploadDir;

    @PostMapping("/uploadFile")
    public String uploadFile(@RequestParam("file") MultipartFile file) {
        System.out.println("上传文件: " + file.getOriginalFilename());
        File directory = new File(uploadDir);
        if (!directory.exists()) {
            directory.mkdirs();
        }
        String fileName = file.getOriginalFilename(); // 获取文件名包括后缀
        String filePath = uploadDir + File.separator + fileName;
        File destFile = new File(filePath);

        // 检查是否存在同名文件，包括后缀
        if (destFile.exists()) {
            return "上传失败: 文件已存在";
        }
        try {
            file.transferTo(new File(filePath));
            return "上传成功!";
        } catch (IOException e) {
            return "上传失败" + e.getMessage();
        }
    }

    @GetMapping("/listFile")
    public ResponseEntity<List<String>> listUploadedFiles() {
        File directory = new File(uploadDir);
        List<String> fileNames = new ArrayList<>();

        if (directory.exists() && directory.isDirectory()) {
            File[] files = directory.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isFile()) {
                        fileNames.add(file.getName());
                    }
                }
            }
        } else {
            fileNames.add("目录不存在或不是一个目录");
        }
        System.out.println("文件列表：" + fileNames);
//        return fileNames;
        return ResponseEntity.ok().contentType(MediaType.APPLICATION_JSON_UTF8).body(fileNames);
    }

    @PostMapping("/deleteFile")
    public String deleteFile(String fileName) {
        File file = new File(uploadDir + File.separator + fileName);
        if (!file.exists()) {
            return "删除失败: 文件不存在";
        }

        if (file.delete()) {
            return "删除成功!";
        } else {
            return "删除失败: 无法删除文件";
        }
    }

    @PostMapping("/downloadFile")
    public ResponseEntity<Object> downloadFile(String fileName) {
        File file = new File(uploadDir, fileName);
        if (!file.exists()) {
            return new ResponseEntity<>("文件不存在", HttpStatus.NOT_FOUND);
        }
        try {
            InputStreamResource resource = new InputStreamResource(new FileInputStream(file));
            HttpHeaders headers = new HttpHeaders();
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
//                    .replaceAll("%2B", "\\+");
            String encodedFileNameNew = URLDecoder.decode(encodedFileName, StandardCharsets.UTF_8.toString());
            System.out.println("encodedFileNameNew: " + encodedFileNameNew);
            headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + encodedFileName + "\"");
            headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE);
            return ResponseEntity.ok()
                    .headers(headers)
                    .contentLength(file.length())
                    .contentType(MediaType.APPLICATION_OCTET_STREAM)
                    .body(resource);

        } catch (FileNotFoundException e) {
            return new ResponseEntity<>("文件未找到: " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
        } catch (IOException e) {
            return new ResponseEntity<>("下载文件时出错: " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}